package main

import (
	"AT-Server/server"
	"AT-Server/services"
	"context"
	"fmt"
	"github.com/gogf/gf/frame/g"
	"github.com/gogf/gf/os/glog"
	"github.com/gogf/gf/os/gproc"
	rpcx "github.com/smallnest/rpcx/server"
	"os"
)

func main() {
	err := glog.SetLevelStr(g.Cfg().GetString("Server.RunMode"))
	if err != nil {
		panic(err)
	}
	ctx := context.Background()
	srv := server.NewServer(
		ctx,
		g.Cfg().GetString("Server.Addr"),
		g.Cfg().GetInt("Server.Port"),
	)
	go func() {
		_ = srv.Start()
	}()
	s := rpcx.NewServer()
	if err = s.Register(services.NewATService(srv), ""); err != nil {
		panic(err)
	}
	go func() {
		addr := g.Cfg().GetString("RPC.Addr")
		port := g.Cfg().GetInt("RPC.Port")
		glog.Printf("RPC服务启动: [%s:%d]", addr, port)
		_ = s.Serve("tcp", fmt.Sprintf("%s:%d", addr, port))
	}()

	gproc.AddSigHandlerShutdown(func(sig os.Signal) {
		_ = srv.Stop()
		_ = s.Shutdown(ctx)
	})
	gproc.Listen()
}
